/////////////////////////////////////////////////////////////////////////////
//
//  sbs-cm.asl - Smart Battery System Control Method Interface ($Revision: 1.10 $)
//
//  (C) 2005 Rich Townsend <rhdt@bartol.udel.edu>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or (at
//  your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
//
//  $Log: sbs-cm-2b.asl,v $
//  Revision 1.10  2005/02/13 04:49:08  rhdt
//  Fixed miscalculation of power rate in UPBS () routine
//
//  Revision 1.9  2005/02/11 19:33:27  rhdt
//  Fixed bug in UPBI () method, relating to the scaling of the last full
//  capacity field
//
//  Revision 1.8  2005/02/11 04:55:40  rhdt
//  Removed redundant debugging output
//
//  Revision 1.7  2005/02/11 04:52:40  rhdt
//  Introduced a new SLEV () method, for handling selector events, and
//  rewrote the _Q20 () SMBus alarm handling method
//
//  Revision 1.6  2005/02/08 00:27:06  rhdt
//  Renamed SECB () method to ACQB (), and added in a complementary RELB ()
//  release method
//
//  Revision 1.5  2005/02/07 15:26:31  rhdt
//  Fixed regression in both STA_ () methods
//
//  Revision 1.4  2005/02/07 15:20:31  rhdt
//  Fixed bug in _STA () method of BAT1
//
//  Revision 1.3  2005/02/07 15:18:46  rhdt
//  Streamlined the battery selection code, with a new SECB () method
//
//  Revision 1.2  2005/02/07 14:51:49  rhdt
//  Fixed calculation of present power rate in UPBS ()
//
//  Revision 1.1.1.1  2005/02/04 02:04:35  rhdt
//  Imported sources
//
//
/////////////////////////////////////////////////////////////////////////////

Scope (\_SB)
{

    Mutex (BAMX, 0)         // Battery selection mutex

    //
    // ACQB (Acquire Battery)
    // ----------------------
    //
    // Arguments:
    //  0 - UID (integer, 0-3)
    //
    // Return: (integer)
    //  Status (0=failure, 1=success + battery mutex acquired)
    //
    Method (ACQB, 1, NotSerialized)
    {
        If (LNot (Acquire (BAMX, 0xFFFF)))
        {
            Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0A, 0x01), Local0)

            Or (ShiftLeft (1, Add (Arg0, 12)), And(Local0, 0x0FFF), Local1)
            \_SB.PCI0.LPC0.EC0.SMB0.SBWW (0x0A, 0x01, Local1)

            Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0A, 0x01), Local0)
            If (LEqual (And (Local0, 0xF000), And (Local1, 0xF000)))
            {
                Return (1)
            }

            Release (BAMX)
        }

        Return (0)
    } // ACQB ()

    //
    // RELB (Release Battery)
    // ----------------------
    //
    // Arguments:
    //  None
    //
    // Return:
    //  None
    //
    Method (RELB, 0, NotSerialized)
    {
        Release (BAMX)
    }
        
/////////////////////////////////////////////////////////////////////////////
// Device: BAT0
/////////////////////////////////////////////////////////////////////////////

    Device (BAT0)
    {
        Name (_HID, EisaId ("PNP0C0A"))     // Hardware ID   
        Name (_UID, 0x00)                   // Unique ID

        Name (_PCL, Package (0x01)          // Power Consumer List
        {
            \_SB
        })
        
        Name (PBIF, Package (0x0D)  // Battery info package
        {
            0x01,                   // Power unit
            0xFFFFFFFF,             // Design capacity
            0xFFFFFFFF,             // Last full charge capacity
            0x01,                   // Battery technology
            0xFFFFFFFF,             // Design voltage
            0x00,                   // Design capacity of warning
            0x00,                   // Design capacity of low
            0x00,                   // Battery capacity granularity 1
            0x00,                   // Battery capacity granularity 2
            "BAT0",                 // Model number
            " ",                    // Serial number
            " ",                    // Battery type
            " "                     // OEM information
        })

        Name (PBST, Package (0x04)  // Battery status package
        {
            0x00,                   // Battery state flags
            0xFFFFFFFF,             // Battery present rate
            0xFFFFFFFF,             // Battery remaining capacity
            0xFFFFFFFF              // Battery present voltage
        })

        Name (BAST, 0x00)           // Battery status flags
        Name (BAAL, 0x00)           // Battery alarm flags

        //
        // _STA (Status)
        // -------------
        // 
        // Arguments:
        //  None
        //
        // Return: (integer)
        //  Battery status
        //
        Method (_STA, 0, NotSerialized)
        {
            If (\_SB.ECOK)
            {
                Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0A, 0x01), Local1)

                If (And (Local1, 0x01))
                {
                    If (LNotEqual (BAST, 0x1F))    // Check for first initialization
                    {
                        Store (0x1F, BAST)
                        SEBI ()
                    }
                }
                Else
                {
                    Store (0x0F, BAST)
                }
            }
            Else
            {
                Store (0x0F, BAST)
            }

            Return (BAST)
        } // _STA ()

        //
        // _BIF (Battery Info)
        // -------------------
        //
        // Arguments:
        //  None
        //
        // Return: (package)
        //  Battery info
        //
        Method (_BIF, 0, NotSerialized)
        {
            Return (PBIF)
        } // _BIF ()

        //
        // _BST (Battery Status)
        // ---------------------
        //
        // Arguments:
        //  None
        //
        // Return: (package)
        //  Battery status
        //
        Method (_BST, 0, NotSerialized)
        {
            SEBS ()
            Return (PBST)
        } // _BST ()

        //
        // _BTP (Battery Trip Point)
        // -------------------------
        //
        // Arguments:
        //  0 - Trip point (integer)
        //
        // Return:
        //  None
        //
        Method (_BTP, 1, NotSerialized)
        {
            If (And (BAST, 0x10)) {
                If (ACQB (_UID))
                {
                    \_SB.PCI0.LPC0.EC0.SMB0.SBWW (0x0B, 0x01, Arg0)     // Capacity alarm
                    \_SB.PCI0.LPC0.EC0.SMB0.SBWW (0x0B, 0x02, 0x00)    	// Time alarm (disabled)
            
                    Store(\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0B, 0x03), Local1)
                    \_SB.PCI0.LPC0.EC0.SMB0.SBWW (0x0B, 0x03, And (Local1, 0xC300)) // Enable (0) the ALARM_MODE flag

                    RELB ()
                }
            }

        } // _BTP ()

        //
        // SEBI (Set Battery Info)
        // -----------------------
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (SEBI, 0, NotSerialized)
        {
            If (And (BAST, 0x10))
    	    {
                UPBI () // Update the battery information package
            }
            Else
            {   
                IVBI () // Invalidate the battery information package
            }
        } // SEBI ()
            
        //
        // SEBS (Set Battery Status)
        // -------------------------
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (SEBS, 0, NotSerialized)
        {
            If (And (BAST, 0x10))
    	    {
                UPBS () // Update the battery status package
            }
            Else
            {   
                IVBS () // Invalidate the battery status package
            }
        } // SEBS ()
            
        //
        // UPBI (Update Battery Information)
        // ---------------------------------
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (UPBI, 0, NotSerialized)
        {
            Store (Buffer (0x0D)
            {
                0x03, 0x18, 0x10, 0x00, 0x19, 0x00, 0x00, 
                0x00, 0x00, 0x21, 0x00, 0x22, 0x20
            }, Local0)

            If (ACQB (_UID))
            {
                // Grab data from the smart battery

                Store (0x00, Local1)
                While (LLess (Local1, 0x0D))
                {
                    Store (DeRefOf (Index (Local0, Local1)), Local2)
                    If (Local2)
                    {
                        If (LLess (Local1, 0x0A))
                        {
                            Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0B, Local2), Index(PBIF, Local1))    // Word field
                        }
                        Else
                        {
                            Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRS (0x0B, Local2), Index(PBIF, Local1))    // String field
                        }
                    }
                    Increment (Local1)
                }

                // Fix up those fields that require it

                Store (DeRefOf (Index (PBIF, 0)), Local1)
                If (And (Local1, 0x8000))
                {
                    Store (0x00, Index (PBIF, 0))           // Power unit mW[h]

                    Store (DeRefOf (Index (PBIF, 2)), Local2)
                    Multiply (Local2, 10, Index (PBIF, 2))

                    Store (DeRefOf (Index (PBIF, 1)), Local2)
                    Multiply (Local2, 10, Index (PBIF, 1))
                    Store (Local2, Index (PBIF, 5))         // Low capacity 10%
                    Divide (Local2, 2, , Index (PBIF, 6))   // Warning capacity 5%

                    Store (10, Index (PBIF, 7))             // Granularity 1
                    Store (10, Index (PBIF, 8))             // Granularity 2
                }
                Else
                {
                    Store (0x01, Index (PBIF, 0))           // Power unit mA[h]

                    Store (DeRefOf (Index (PBIF, 1)), Local2)
                    Divide (Local2, 10, , Index (PBIF, 5))  // Low capacity 10%
                    Divide (Local2, 20, , Index (PBIF, 6))  // Warning capacity 5%

                    Store (1, Index (PBIF, 7))              // Granularity 1
                    Store (1, Index (PBIF, 8))              // Granularity 2
                }
                    
                If (LEqual (And (Local1, 0x0202), 0x0202))
                {
                    Store (0x00, Index (PBIF, 3)) // Battery technology primary
                }
                Else
                {
                    Store (0x01, Index (PBIF, 3)) // Battery technology secondary
                }

                // Set up the serial number

                Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0B, 0x1C), Local1)
                ToDecimalString (Local1, Index (PBIF, 10))

                RELB ()
            }

        } // UPBI ()

        //
        // UPBS (Update Battery Status)
        // ----------------------------    
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (UPBS, 0, NotSerialized)
        {
            Store (Buffer (0x0D)
            {
                0x00, 0x0A, 0x0F, 0x09
            }, Local0)

            If (ACQB (_UID))
            {
                // Grab data from the smart battery

                Store (0x00, Local1)
                While (LLess (Local1, 0x04))
                {
                    Store (DeRefOf (Index (Local0, Local1)), Local2)
                    If (Local2)
                    {
                        Store (DeRefOf (Index (Local0, Local1)), Local2)
                        Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0B, Local2), Index(PBST, Local1)) // Word field
                    }
                    Increment (Local1)
                }

                // Fix up those fields that require it

                Store (DeRefOf (Index (PBIF, 0)), Local1) 
                Store (WABS (DeRefOf (Index (PBST, 1))), Local2)
                If (LNot (Local1))
                {
                    Multiply ( Local2, DeRefOf (Index (PBST, 3)), Local2)   // Power = current*voltage
                    Divide (Local2, 1000, , Index (PBST, 1))
                    Multiply (DeRefOf (Index (PBST, 2)), 10, Index (PBST, 2))
                }
                Else
                {
                    Store (Local2, Index (PBST, 1))
                }

                // Set the battery state flags

    		    Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0B, 0x16), Local1)

                Store (0x00, Local2)

                If (\_SB.ACAD._PSR)
                {
                    If (LNot( And (Local1, 0x40)))
                    {
                        Or (Local2, 0x02, Local2)
                    }
                }
                Else
                {
                    If (DeRefOf (Index (PBST, 1)))
                    {
                        Or (Local2, 0x01, Local2)
                    }
                }

                Store (Local2, Index (PBST, 0))

                RELB ()
            }
            
        } // UPBS ()              

        //
        // IVBI (Invalidate Battery Information)
        // -------------------------------------
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (IVBI, 0, NotSerialized)
        {
            // Only invalidate those numeric fields that have
            // spec.-defined invaid values 

            Store (0xFFFFFFFF, Index (PBIF, 1))
            Store (0xFFFFFFFF, Index (PBIF, 2))
            Store (0xFFFFFFFF, Index (PBIF, 4))

            // Also invalidate the string fields

            Store ("Invalid", Index (PBIF, 9))
            Store ("Invalid", Index (PBIF, 10))
            Store ("Invalid", Index (PBIF, 11))
            Store ("Invalid", Index (PBIF, 12))

        } // IVBI()
            
        //
        // IVBS (Invalidate Battery Status)
        // --------------------------------
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (IVBS, 0, NotSerialized)
        {
            // Only invalidate those numeric fields that have
            // spec.-defined invaid values 

            Store (0xFFFFFFFF, Index (PBST, 1))
            Store (0xFFFFFFFF, Index (PBST, 2))

        } // IVBI()

        //
        // CKAL (Check alarm)
        // ------------------
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (CKAL, 0, NotSerialized)
        {    
            If (ACQB (_UID))
            {
                Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0B, 0x16), Local0)
                And (Local0, 0x0300, Local0)    // Capacity/time alarm bits

                If(LNotEqual (Local0, BAAL))
                {
                    Store (Local0, BAAL)
                    Notify (\_SB.BAT0, 0x80)
                }

                RELB ()
            }
        } // CKAL ()

    } // Device (BAT0)

/////////////////////////////////////////////////////////////////////////////
// Device: BAT1
/////////////////////////////////////////////////////////////////////////////

    Device (BAT1)
    {
        Name (_HID, EisaId ("PNP0C0A"))     // Hardware ID   
        Name (_UID, 0x01)                   // Unique ID

        Name (_PCL, Package (0x01)          // Power Consumer List
        {
            \_SB
        })
        
        Name (PBIF, Package (0x0D)  // Battery info package
        {
            0x01,                   // Power unit
            0xFFFFFFFF,             // Design capacity
            0xFFFFFFFF,             // Last full charge capacity
            0x01,                   // Battery technology
            0xFFFFFFFF,             // Design voltage
            0x00,                   // Design capacity of warning
            0x00,                   // Design capacity of low
            0x00,                   // Battery capacity granularity 1
            0x00,                   // Battery capacity granularity 2
            "BAT1",                 // Model number
            " ",                    // Serial number
            " ",                    // Battery type
            " "                     // OEM information
        })

        Name (PBST, Package (0x04)  // Battery status package
        {
            0x00,                   // Battery state flags
            0xFFFFFFFF,             // Battery present rate
            0xFFFFFFFF,             // Battery remaining capacity
            0xFFFFFFFF              // Battery present voltage
        })

        Name (BAST, 0x00)           // Battery status flags
        Name (BAAL, 0x00)           // Battery alarm flags

        //
        // _STA (Status)
        // -------------
        // 
        // Arguments:
        //  None
        //
        // Return: (integer)
        //  Battery status
        //
        Method (_STA, 0, NotSerialized)
        {
            If (\_SB.ECOK)
            {
                Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0A, 0x01), Local1)

                If (And (Local1, 0x02))
                {
                    If (LNotEqual (BAST, 0x1F))    // Check for first initialization
                    {
                        Store (0x1F, BAST)
                        SEBI ()
                    }
                }
                Else
                {
                    Store (0x0F, BAST)
                }
            }
            Else
            {
                Store (0x0F, BAST)
            }

            Return (BAST)
        } // _STA ()

        //
        // _BIF (Battery Info)
        // -------------------
        //
        // Arguments:
        //  None
        //
        // Return: (package)
        //  Battery info
        //
        Method (_BIF, 0, NotSerialized)
        {
            Return (PBIF)
        } // _BIF ()

        //
        // _BST (Battery Status)
        // ---------------------
        //
        // Arguments:
        //  None
        //
        // Return: (package)
        //  Battery status
        //
        Method (_BST, 0, NotSerialized)
        {
            SEBS ()
            Return (PBST)
        } // _BST ()

        //
        // _BTP (Battery Trip Point)
        // -------------------------
        //
        // Arguments:
        //  0 - Trip point (integer)
        //
        // Return:
        //  None
        //
        Method (_BTP, 1, NotSerialized)
        {
            If (And (BAST, 0x10)) {
                If (ACQB (_UID))
                {
                    \_SB.PCI0.LPC0.EC0.SMB0.SBWW (0x0B, 0x01, Arg0)     // Capacity alarm
                    \_SB.PCI0.LPC0.EC0.SMB0.SBWW (0x0B, 0x02, 0x00)    	// Time alarm (disabled)
            
                    Store(\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0B, 0x03), Local1)
                    \_SB.PCI0.LPC0.EC0.SMB0.SBWW (0x0B, 0x03, And (Local1, 0xC300)) // Enable (0) the ALARM_MODE flag

                    RELB ()
                }
            }

        } // _BTP ()

        //
        // SEBI (Set Battery Info)
        // -----------------------
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (SEBI, 0, NotSerialized)
        {
            If (And (BAST, 0x10))
    	    {
                UPBI () // Update the battery information package
            }
            Else
            {   
                IVBI () // Invalidate the battery information package
            }
        } // SEBI ()
            
        //
        // SEBS (Set Battery Status)
        // -------------------------
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (SEBS, 0, NotSerialized)
        {
            If (And (BAST, 0x10))
    	    {
                UPBS () // Update the battery status package
            }
            Else
            {   
                IVBS () // Invalidate the battery status package
            }
        } // SEBS ()
            
        //
        // UPBI (Update Battery Information)
        // ---------------------------------
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (UPBI, 0, NotSerialized)
        {
            Store (Buffer (0x0D)
            {
                0x03, 0x18, 0x10, 0x00, 0x19, 0x00, 0x00, 
                0x00, 0x00, 0x21, 0x00, 0x22, 0x20
            }, Local0)

            If (ACQB (_UID))
            {
                // Grab data from the smart battery

                Store (0x00, Local1)
                While (LLess (Local1, 0x0D))
                {
                    Store (DeRefOf (Index (Local0, Local1)), Local2)
                    If (Local2)
                    {
                        If (LLess (Local1, 0x0A))
                        {
                            Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0B, Local2), Index(PBIF, Local1))    // Word field
                        }
                        Else
                        {
                            Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRS (0x0B, Local2), Index(PBIF, Local1))    // String field
                        }
                    }
                    Increment (Local1)
                }

                // Fix up those fields that require it

                Store (DeRefOf (Index (PBIF, 0)), Local1)
                If (And (Local1, 0x8000))
                {
                    Store (0x00, Index (PBIF, 0))           // Power unit mW[h]

                    Store (DeRefOf (Index (PBIF, 2)), Local2)
                    Multiply (Local2, 10, Index (PBIF, 2))

                    Store (DeRefOf (Index (PBIF, 1)), Local2)
                    Multiply (Local2, 10, Index (PBIF, 1))
                    Store (Local2, Index (PBIF, 5))         // Low capacity 10%
                    Divide (Local2, 2, , Index (PBIF, 6))   // Warning capacity 5%

                    Store (10, Index (PBIF, 7))             // Granularity 1
                    Store (10, Index (PBIF, 8))             // Granularity 2
                }
                Else
                {
                    Store (0x01, Index (PBIF, 0))           // Power unit mA[h]

                    Store (DeRefOf (Index (PBIF, 1)), Local2)
                    Divide (Local2, 10, , Index (PBIF, 5))  // Low capacity 10%
                    Divide (Local2, 20, , Index (PBIF, 6))  // Warning capacity 5%

                    Store (1, Index (PBIF, 7))              // Granularity 1
                    Store (1, Index (PBIF, 8))              // Granularity 2
                }
                    
                If (LEqual (And (Local1, 0x0202), 0x0202))
                {
                    Store (0x00, Index (PBIF, 3)) // Battery technology primary
                }
                Else
                {
                    Store (0x01, Index (PBIF, 3)) // Battery technology secondary
                }

                // Set up the serial number

                Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0B, 0x1C), Local1)
                ToDecimalString (Local1, Index (PBIF, 10))

                RELB ()
            }

        } // UPBI ()

        //
        // UPBS (Update Battery Status)
        // ----------------------------    
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (UPBS, 0, NotSerialized)
        {
            Store (Buffer (0x0D)
            {
                0x00, 0x0A, 0x0F, 0x09
            }, Local0)

            If (ACQB (_UID))
            {
                // Grab data from the smart battery

                Store (0x00, Local1)
                While (LLess (Local1, 0x04))
                {
                    Store (DeRefOf (Index (Local0, Local1)), Local2)
                    If (Local2)
                    {
                        Store (DeRefOf (Index (Local0, Local1)), Local2)
                        Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0B, Local2), Index(PBST, Local1)) // Word field
                    }
                    Increment (Local1)
                }

                // Fix up those fields that require it

                Store (DeRefOf (Index (PBIF, 0)), Local1) 
                Store (WABS (DeRefOf (Index (PBST, 1))), Local2)
                If (LNot (Local1))
                {
                    Multiply ( Local2, DeRefOf (Index (PBST, 3)), Local2)   // Power = current*voltage
                    Divide (Local2, 1000, , Index (PBST, 1))
                    Multiply (DeRefOf (Index (PBST, 2)), 10, Index (PBST, 2))
                }
                Else
                {
                    Store (Local2, Index (PBST, 1))
                }

                // Set the battery state flags

    		    Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0B, 0x16), Local1)

                Store (0x00, Local2)

                If (\_SB.ACAD._PSR)
                {
                    If (LNot( And (Local1, 0x40)))
                    {
                        Or (Local2, 0x02, Local2)
                    }
                }
                Else
                {
                    If (DeRefOf (Index (PBST, 1)))
                    {
                        Or (Local2, 0x01, Local2)
                    }
                }

                Store (Local2, Index (PBST, 0))

                RELB ()
            }
            
        } // UPBS ()              

        //
        // IVBI (Invalidate Battery Information)
        // -------------------------------------
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (IVBI, 0, NotSerialized)
        {
            // Only invalidate those numeric fields that have
            // spec.-defined invaid values 

            Store (0xFFFFFFFF, Index (PBIF, 1))
            Store (0xFFFFFFFF, Index (PBIF, 2))
            Store (0xFFFFFFFF, Index (PBIF, 4))

            // Also invalidate the string fields

            Store ("Invalid", Index (PBIF, 9))
            Store ("Invalid", Index (PBIF, 10))
            Store ("Invalid", Index (PBIF, 11))
            Store ("Invalid", Index (PBIF, 12))

        } // IVBI()
            
        //
        // IVBS (Invalidate Battery Status)
        // --------------------------------
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (IVBS, 0, NotSerialized)
        {
            // Only invalidate those numeric fields that have
            // spec.-defined invaid values 

            Store (0xFFFFFFFF, Index (PBST, 1))
            Store (0xFFFFFFFF, Index (PBST, 2))

        } // IVBI()

        //
        // CKAL (Check alarm)
        // ------------------
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (CKAL, 0, NotSerialized)
        {    
            If (ACQB (_UID))
            {
                Store (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x0B, 0x16), Local0)
                And (Local0, 0x0300, Local0)    // Capacity/time alarm bits

                If(LNotEqual (Local0, BAAL))
                {
                    Store (Local0, BAAL)
                    Notify (\_SB.BAT1, 0x80)
                }

                RELB ()
            }
        } // CKAL ()

    } // Device (BAT1)

/////////////////////////////////////////////////////////////////////////////
// Device: ACAD
/////////////////////////////////////////////////////////////////////////////

    Device (ACAD)
    {
        Name (_HID, "ACPI0003")             // Hardware ID   
        Name (_UID, 0x00)                   // Unique ID

        Name (_PCL, Package (0x01)          // Power Consumer List
        {
            \_SB
        })
        
        //
        // _PSR (Power Source)
        // -------------------
        //
        // Arguments:
        //  None
        //
        // Return:
        //  None
        //
        Method (_PSR, 0, NotSerialized)
        {
            If (\_SB.ECOK)
            {
                If (And (\_SB.PCI0.LPC0.EC0.SMB0.SBRW (0x09, 0x13), 0x8000))
                {
                    Store (0x01, Local0)
                }
                Else
                {
                    Store (0x00, Local0)
                }
            }
            Return (Local0)
        } // _PSR()
            
    } // Device(ACAD)

}

/////////////////////////////////////////////////////////////////////////////
// Embedded Controller hooks
/////////////////////////////////////////////////////////////////////////////

Scope (\_SB.PCI0.LPC0.EC0)
{

    Name (PSST, 0x00)   // Previous selector state

    //
    // SLEV (Selector Event)
    // ---------------------
    //
    // Arguments:
    //  0 - Selector state (integer)
    //
    // Return: (integer)
    //  Event flag:
    //    bit 0 - battery A inserted/removed
    //    bit 1 - battery B intsered/removed
    //    bit 2 - battery C intsered/removed
    //    bit 3 - battery D intsered/removed
    //    bit 4 - AC inserted/removed
    //
    Method (SLEV, 1, NotSerialized)
    {
        Store (0x00, Local0)

        Store (0x00, Local1) 
        Store (0x00, Local3)
        Store (0x00, Local4)
        While (LLess (Local1, 4))
        {
            ShiftLeft(1, Local1, Local2)

            If (XOr (And (Arg0, Local2), And (PSST, Local2))) // Battery inserted/removed
            {
                Or (Local0, Local2, Local0)
            }

            ShiftLeft(1, Add (Local1, 4), Local2)

            If (And (Arg0, Local2)) // Battery charger bit set
            {
                Increment (Local3)
            }

            If (And (PSST, Local2)) // Battery charger bit set
            {
                Increment (Local4)
            }

            Increment (Local1)
        }

        If (XOr (LLess(Local3, 2), LLess(Local4, 2))) 
        {
            Or (Local0, 0x10, Local0)
        }

        Store (Arg0, PSST) // Save the selector state

        Return (Local0)
    } // SLEV ()

    //
    // _Q09 (BPOL Polling Handler)
    // ---------------------------
    //
    // Arguments:
    //  None
    //
    // Return:
    //  None
    //
    Method (_Q09, 0, NotSerialized)
    {
        If (And (\_SB.BAT0.BAST, 0x10))
        {
            \_SB.BAT0.CKAL ()
        }

        If (And (\_SB.BAT1.BAST, 0x10))
        {
            \_SB.BAT1.CKAL ()
        }

        BPOL (0x3C) // Set the next poll to take place a minute hence
    } // _Q09 ()

    //
    // _Q20 (SMBus Alarm Handler)
    // --------------------------
    //
    // Arguments:
    //  None
    //
    // Return:
    //  None
    //
    Method (_Q20, 0, NotSerialized)
    {
        Store (\_SB.PCI0.LPC0.EC0.SMB0._SBA (), Local0)
        Store (DeRefOf (Index (Local0, 0)), Local1) // Status code
        Store (DeRefOf (Index (Local0, 1)), Local2) // Slave address
        Store (DeRefOf (Index (Local0, 3)), Local3) // Data

        If (LNot (Local1))
        {
            If (LEqual (Local2, 0x0A))
            {
                Store (SLEV (Local3), Local4)

                If (And (Local4, 0x01))   // Battery A inserted/removed...
                {
                    \_SB.BAT0.SEBI ()
                    Notify (\_SB.BAT0, 0x00)
                    Notify (\_SB.BAT0, 0x81)
                }

                If (And (Local4, 0x02))   // Battery B inserted/removed...
                {
                    \_SB.BAT1.SEBI ()
                    Notify (\_SB.BAT1, 0x00)
                    Notify (\_SB.BAT1, 0x81)
                }

                If (And (Local4, 0x10))   // AC status changed...
                {
                    Notify (\_SB.ACAD, 0x80)
                }

                Notify (\_SB.BAT0, 0x80) 
                Notify (\_SB.BAT1, 0x80) 
            }
            ElseIf (Local2)
            {
                Store ("_Q20: Unrecognized SMBus event:", Debug)
                Store (Local1, Debug)
                Store (Local2, Debug)
                Store (Local3, Debug)
            }
        }
    } // _Q20 ()
}

/////////////////////////////////////////////////////////////////////////////
// Miscellaneous routines
/////////////////////////////////////////////////////////////////////////////

Scope (\_SB)
{
    //
    // WABS (Word Absolute Value)
    // --------------------------
    //
    // Parameters:
    //  0 - word (integer)
    //
    // Return: (integer)
    //  Absolute value of word 
    //
    Method (WABS, 1, NotSerialized)
    {
        If (And (Arg0, 0x8000))                 // If the word < 0...
        {
            If (LEqual (Arg0, 0xFFFF))          // If the word is a special flag value...
            {
                Return (0xFFFFFFFF)             // ...return an appropriate value
            }
            Else
            {
                Not (Arg0, Local0)              // Store the inverse of Arg0 in Local0
                Increment (Local0)              // Add one to Local0 (for two's complement)
                And (Local0, 0xFFFF, Local0)    // Mask out the high bits
                Return (Local0)                 // Return the result
            }
        }
        Else
        {
            Return (Arg0)                       // Return the word as is
        }

    } // WABS ()

}
