/////////////////////////////////////////////////////////////////////////////
//
//  smbus-cm.asl - System Management Bus Control Method Interface ($Revision: 1.3 $)
//
//  (C) 2005 Rich Townsend <rhdt@bartol.udel.edu>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or (at
//  your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
//
//  $Log: smbus-cm.asl,v $
//  Revision 1.3  2005/03/08 03:06:14  rhdt
//  Modified _SBA () method, to prevent mutex thrashing
//
//  Revision 1.2  2005/02/11 04:51:34  rhdt
//  Added new _SBA () method
//
//  Revision 1.1.1.1  2005/02/04 02:04:33  rhdt
//  Imported sources
//
//
/////////////////////////////////////////////////////////////////////////////

Scope (\_SB.PCI0.LPC0.EC0)
{

/////////////////////////////////////////////////////////////////////////////
// Device: SMB0
/////////////////////////////////////////////////////////////////////////////

    Device (SMB0)
    {
        Name (_HID, "SBMUS01")  // Hardward ID   
        Name (_UID, 0x00)       // Unique ID

        Mutex (SBMX, 0)         // SMBus mutex

        OperationRegion (SMB0, EmbeddedControl, 0x18, 0x40)
        Field (SMB0, ByteAcc, Lock, Preserve)
        {
            SBPR, 8,    // Protocol
            SBST, 8,    // Status
            SBAD, 8,    // Address
            SBCM, 8,    // Command
            SBDA, 256,  // Data
            SBBC, 8,    // Block count
            SBAA, 8,    // Alarm address
            SBAW, 16,   // Alarm data word
        }
        Field (SMB0, ByteAcc, Lock, Preserve)
        {
            Offset (0x04),
            SBDB, 8     // Data byte
        }

        Field (SMB0, ByteAcc, Lock, Preserve)
        {
            Offset (0x04),
            SBDW, 16    // Data word
        }

        //
        // _SBI (SMBus Information)
        // ------------------------
        //
        // Arguments:
        //  None
        //
        // Return: (package)
        //  (0) - SMBus CMI version (integer)
        //  (1) - SMBus information (buffer)
        //
        Method (_SBI, 0, NotSerialized)
        {
            Store(Package (0x02){0x10,0x00}, Local0)

            Store(Buffer ()
            {
                0x10,       // SMB_INFO structure version (1.0)
                0x10,       // SMBus specification version (1.0)
                0x00,       // Segment hardware capability (no PEC, no ARP)
                0x00,       // Alert polling interval (asyncrhonous)
                0x04,       // Device count (3)
                0x09, 0x00, //   SBM_UDID (SBS charger)
                0x00, 0x00, //     Device hardware capabilities (no PEC)
                0x00, 0x00, //     Version/revision
                0xFF, 0xFF, //     Vendor ID
                0x00, 0x00, //     Device ID
                0x00, 0x00, //     Interface (1.0)
                0x53, 0x42, //     Subsystem vendor ID (SBS-IF)
                0x53, 0x09, //     Subsystem device ID (SBS charger)
                0x00, 0x00, //     Reserved
                0x00, 0x00, //     Reserved
                0x0A, 0x00, //   SBM_UDID (SBS system manager/selector)
                0x00, 0x00, //     Device hardware capabilities (no PEC)
                0x00, 0x00, //     Version/revision
                0xFF, 0xFF, //     Vendor ID
                0x00, 0x00, //     Device ID
                0x00, 0x00, //     Interface (1.0)
                0x53, 0x42, //     Subsystem vendor ID (SBS-IF)
                0x53, 0x0A, //     Subsystem device ID (SBS selector)
                0x00, 0x00, //     Reserved
                0x00, 0x00, //     Reserved
                0x0B, 0x00, //   SBM_UDID (SBS battery)
                0x00, 0x00, //     Device hardware capabilities (no PEC)
                0x00, 0x00, //     Version/revision
                0xFF, 0xFF, //     Vendor ID
                0x00, 0x00, //     Device ID
                0x00, 0x00, //     Interface (1.0)
                0x53, 0x42, //     Subsystem vendor ID (SBS-IF)
                0x53, 0x0B, //     Subsystem device ID (SBS battery)
                0x00, 0x00, //     Reserved
                0x00, 0x00  //     Reserved
            }, Index (Local0, 1))

            Return (Local0)
        } // SBI_ ()

        //
        // _SBR (SMBus Data Read)
        // ----------------------
        //
        // Arguments:
        //  (0) - Protocol value (integer)
        //  (1) - Slave address (integer)
        //  (2) - Command code (integer)
        //
        // Return: (package)
        //  (0) - Status code (integer)
        //  (1) - Data length (integer)
        //  (2) - Data (integer | buffer)
        //
        Method (_SBR, 3, NotSerialized)
        {
            If (LEqual (Arg0, 0x0B))
            {
                Store (Package (0x03){0x07,0x00,Buffer(0x20){}}, Local0)
            }
            Else
            {
                Store (Package (0x03){0x07,0x00,0x00}, Local0)
            }

            // Check the protocol value

            If (LNotEqual (Arg0, 0x07))
            {
                If (LNotEqual (Arg0, 0x09))
                {
                    If (LNotEqual (Arg0, 0x0B))
                    {
                        Store (0x19, Index (Local0, 0))
                        Return (Local0)
                    }
                }
            }

            // Acquire the SMBus mutex
        
            If (LNot (Acquire (SBMX, 0xFFFF)))
            {
                // Check the bus status

                If (LNotEqual (SBPR, 0x00))
                {
                    Store (0x1A, Index (Local0, 0))
                }
                Else
                {
                    // Perform the SMBus transaction

                    Store (ShiftLeft (Arg1, 1), SBAD)
                    Store (Arg2, SBCM)
                    Store (Arg0, SBPR)

                    // Wait for completion

                    Store (SBWT (1000), Local1)
                    Store (Local1, Index (Local0, 0))

                    if(LNot (Local1))
                    {
                        If (LEqual (Arg0, 0x07))        // Read byte
                        {
                            Store (0x01, Index (Local0, 1))
                            Store (SBDB, Index (Local0, 2))
                        }
                        ElseIf (LEqual (Arg0, 0x09))    // Read word
                        {
                            Store (0x02, Index (Local0, 1))
                            Store (SBDW, Index (Local0, 2))
                        }
                        ElseIf (LEqual (Arg0, 0x0B))    // Read block
                        {
                            Store (SBBC, Index (Local0, 1))
                            Store (SBDA, Index (Local0, 2))
                        }
                    }
                }

                // Release the mutex

                Release (SBMX)
            }            

            Return (Local0)
        } // _SBR ()

        //
        // _SBW (SMBus Data Write)
        // -----------------------
        //
        // Arguments:
        //  (0) - Protocol value (integer)
        //  (1) - Slave address (integer)
        //  (2) - Command code (integer)
        //  (3) - Data length (integer)
        //  (4) - Data (integer | buffer)
        //
        // Return: (package)
        //  (0) - Status code (integer)
        //
        Method (_SBW, 5, NotSerialized)
        {
            Store(Package (0x01){0x07}, Local0)

            // Check the protocol value

            If(LNotEqual (Arg0, 0x06))
            {
                If(LNotEqual (Arg0, 0x08))
                {
                    If(LNotEqual (Arg0, 0x0A))
                    {
                        Store(0x19, Index (Local0, 0))
                        Return (Local0)
                    }
                }
            }

            // Acquire the SMBus mutex
        
            If (LNot (Acquire (SBMX, 0xFFFF)))
            {
                // Check the bus status

                If (LNotEqual (SBPR, 0x00))
                {
                    Store (0x1A, Index (Local0, 0))
                }
                Else
                {
                    // Perform the SMBus transaction

                    Store (ShiftLeft (Arg1, 1), SBAD)
                    Store (Arg2, SBCM)
                    Store (Arg3, SBBC)

                    If (LEqual (Arg0, 0x06))        // Write byte
                    {
                        Store (Arg4, SBDB)
                    }
                    ElseIf (LEqual (Arg0, 0x08))    // Write word
                    {
                        Store (Arg4, SBDW)
                    }
                    ElseIf (LEqual (Arg0, 0x0A))    // Write block
                    {
                        Store (Arg4, SBDA)
                    }

                    Store (Arg0, SBPR)

                    // Wait for completion

                    Store (SBWT (1000), Local0)
                }

                // Release the mutex

                Release (SBMX)
            }            

            Return (Local0)
        } // _SBW ()

        //
        // _SBA (SMBus Alert Information)
        // ------------------------------
        //
        // Arguments:
        //  None
        //
        // Return: (package)
        //  (0) - Status code (integer)
        //  (1) - Slave address (integer)
        //  (2) - Data length (integer)
        //  (3) - Data (integer)
        //
        Method (_SBA, 0, NotSerialized)
        {
            Store(Package (0x04){0x07,0x00,0x00,0x00}, Local0)

            // Check for an alarm (we do this before the mutex is 
            // acquired, to prevent thrashing)

            If (And (SBST, 0x40))
            {
                // Acquire the SMBus mutex
        
                If (LNot (Acquire (SBMX, 0xFFFF)))
                {
                    Store (0x00, Index (Local0, 0))

                    // Extract the alarm info

                    Store (ShiftRight (SBAA, 1), Index (Local0, 1))
                    Store (2, Index (Local0, 2))
                    Store (SBAW, Index (Local0, 3))

                    // Reset the alarm

                    Store (0x00, SBST)
                }
                Else {
                    Store (0x01, Index (Local0, 0))
                }

                Release (SBMX)
            }

            Return (Local0)
        } // _SBA ()

        //
        // SBWT (SMBus Wait for Transaction)
        // ---------------------------------
        //
        // Arguments:
        //  (0) - Maximum wait in ms (integer)
        //
        // Return: (integer)
        //  0x00 - OK (success)
        //  0x07 - Unknown failure
        //  0x10 - Address not acknowledged
        //  0x11 - Device error
        //  0x12 - Command access denied
        //  0x13 - Unknown error
        //  0x17 - Device access denied
        //  0x18 - Timeout
        //  0x19 - Unsupported protocol
        //  0x1A - Bus busy
        //  0x1F - PEC (CRC-8) error
        //
        Method (SBWT, 1, NotSerialized)
        {
            // Spin until the protocol register becomes zero

            Store (Arg0, Local0)
            While (Local0)
            {
                If (LEqual (SBPR, 0x00))
                {
                    Return (And (SBST, 0x1F))
                }
                Sleep (1)
                Decrement (Local0)
            }

            Return (0x18)
        }

        //
        // SBRW (SMBus Read Word)
        // ----------------------
        //
        // Arguments:
        //  (0) - Slave address (integer)
        //  (1) - Command code (integer)
        //
        // Return: (integer)
        //  Byte read
        //
        Method (SBRW, 2, NotSerialized)
        {
            Store(_SBR (0x09, Arg0, Arg1), Local0)
            Return (DeRefOf( Index (Local0, 2)))
        } // SBRW()

        //
        // SBRS (SMBus Read String)
        // ------------------------
        //
        // Arguments:
        //  (0) - Slave address (integer)
        //  (1) - Command code (integer)
        //
        // Return: (ASCIIZ buffer)
        //  String read
        //
        Method (SBRS, 2, NotSerialized)
        {
            Store(_SBR (0x0B, Arg0, Arg1), Local0)
            Return (ToString( DeRefOf(Index (Local0, 2)), DeRefOf (Index (Local0, 1))))
        } // SBRS ()

        //
        // SBWW (SMBus Write Word)
        // -----------------------
        //
        // Arguments:
        //  (0) - Slave address (integer)
        //  (1) - Command code (integer)
        //  (2) - Data (integer)
        //
        Method (SBWW, 3, NotSerialized)
        {
            Store(_SBW (0x08, Arg0, Arg1, 0x02, Arg2), Local0)
        } // SBWW ()
    }
}
